// © 2022, CGVR (https://cgvr.informatik.uni-bremen.de/),
// Author: Andre Mühlenbrock (muehlenb@uni-bremen.de)

// Include this file only once when compiling:
#pragma once

// Include the std::vector:
#include <vector>

// Include our Mat4f class:
#include "src/math/Mat4f.h"

// Include our Static Mesh Actor class:
#include "src/game/StaticMeshActor.h"

// Include our Mesh class:
#include "src/Mesh.h"

// Include our Shader class:
#include "src/Shader.h"

// Forward declaration of class StaticMeshActor (this is a C++
// specific thing which is required when two classes use each
// other. This is not required in Java, etc.):
class StaticMeshActor;

/**
 * A world class.
 */

struct World {
    // A 'list' which contains all the actors in the world:
    std::vector<StaticMeshActor> actors;

    // A cube mesh which is used to debug render the lights:
    Mesh lightCubeMesh;

    // A shader for rendering light positions and colors:
    Shader lightCubeShader;

    // A list of light colors:
    std::vector<Vec4f> lightColors;

    // Runtime of the scene (assuming 60 FPS):
    float runtime;

    /**
     * Constructs the world with multiple objects.
     */
    World();

    /**
     * Simulates the world, e.g. the time is increased.
     */
    void simulate();

    /**
     * Calls the render method on all actors. Note that this
     * time, we use the same shader for ALL actors and we
     * change it depending on the rendering pass.
     */
    void render(Mat4f view, Mat4f projection, Shader shader);

    /**
     * Binds the given amount of lights to the given shader.
     */
    void bindLights(Mat4f view, Shader shader, int lightNum);

    /**
     * Render icons at the positions of the light.
     */
    void renderLightCubes(Mat4f projection, Mat4f view, int lightNum);
};
